using System;
using System.Text.Json.Serialization;
namespace concerto.metamodel {
   using concerto;
   class Position : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Position";
      public int line { get; set; }
      public int column { get; set; }
      public int offset { get; set; }
   }
   class Range : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Range";
      public Position start { get; set; }
      public Position end { get; set; }
      public string source { get; set; }
   }
   class TypeIdentifier : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.TypeIdentifier";
      public string name { get; set; }
      public string namespace { get; set; }
   }
   abstract class DecoratorLiteral : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DecoratorLiteral";
      public Range location { get; set; }
   }
   class DecoratorString : DecoratorLiteral {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DecoratorString";
      public string value { get; set; }
   }
   class DecoratorNumber : DecoratorLiteral {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DecoratorNumber";
      public float value { get; set; }
   }
   class DecoratorBoolean : DecoratorLiteral {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DecoratorBoolean";
      public bool value { get; set; }
   }
   class DecoratorTypeReference : DecoratorLiteral {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DecoratorTypeReference";
      public TypeIdentifier type { get; set; }
      public bool isArray { get; set; }
   }
   class Decorator : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Decorator";
      public string name { get; set; }
      public DecoratorLiteral[] arguments { get; set; }
      public Range location { get; set; }
   }
   class Identified : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Identified";
   }
   class IdentifiedBy : Identified {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.IdentifiedBy";
      public string name { get; set; }
   }
   abstract class Declaration : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Declaration";
      public string name { get; set; }
      public Decorator[] decorators { get; set; }
      public Range location { get; set; }
   }
   class EnumDeclaration : Declaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.EnumDeclaration";
      public EnumProperty[] properties { get; set; }
   }
   class EnumProperty : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.EnumProperty";
      public string name { get; set; }
      public Decorator[] decorators { get; set; }
      public Range location { get; set; }
   }
   class ConceptDeclaration : Declaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.ConceptDeclaration";
      public bool isAbstract { get; set; }
      public Identified identified { get; set; }
      public TypeIdentifier superType { get; set; }
      public Property[] properties { get; set; }
   }
   class AssetDeclaration : ConceptDeclaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.AssetDeclaration";
   }
   class ParticipantDeclaration : ConceptDeclaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.ParticipantDeclaration";
   }
   class TransactionDeclaration : ConceptDeclaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.TransactionDeclaration";
   }
   class EventDeclaration : ConceptDeclaration {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.EventDeclaration";
   }
   abstract class Property : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Property";
      public string name { get; set; }
      public bool isArray { get; set; }
      public bool isOptional { get; set; }
      public Decorator[] decorators { get; set; }
      public Range location { get; set; }
   }
   class RelationshipProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.RelationshipProperty";
      public TypeIdentifier type { get; set; }
   }
   class ObjectProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.ObjectProperty";
      public string defaultValue { get; set; }
      public TypeIdentifier type { get; set; }
   }
   class BooleanProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.BooleanProperty";
      public bool defaultValue { get; set; }
   }
   class DateTimeProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DateTimeProperty";
   }
   class StringProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.StringProperty";
      public string defaultValue { get; set; }
      public StringRegexValidator validator { get; set; }
   }
   class StringRegexValidator : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.StringRegexValidator";
      public string pattern { get; set; }
      public string flags { get; set; }
   }
   class DoubleProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DoubleProperty";
      public float defaultValue { get; set; }
      public DoubleDomainValidator validator { get; set; }
   }
   class DoubleDomainValidator : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.DoubleDomainValidator";
      public float lower { get; set; }
      public float upper { get; set; }
   }
   class IntegerProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.IntegerProperty";
      public int defaultValue { get; set; }
      public IntegerDomainValidator validator { get; set; }
   }
   class IntegerDomainValidator : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.IntegerDomainValidator";
      public int lower { get; set; }
      public int upper { get; set; }
   }
   class LongProperty : Property {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.LongProperty";
      public long defaultValue { get; set; }
      public LongDomainValidator validator { get; set; }
   }
   class LongDomainValidator : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.LongDomainValidator";
      public long lower { get; set; }
      public long upper { get; set; }
   }
   abstract class Import : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Import";
      public string namespace { get; set; }
      public string uri { get; set; }
   }
   class ImportAll : Import {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.ImportAll";
   }
   class ImportType : Import {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.ImportType";
      public string name { get; set; }
   }
   class Model : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Model";
      public string namespace { get; set; }
      public string sourceUri { get; set; }
      public string concertoVersion { get; set; }
      public Import[] imports { get; set; }
      public Declaration[] declarations { get; set; }
   }
   class Models : Concept {
      [JsonPropertyName("$class")]
      public new string _class { get;} = "concerto.metamodel.Models";
      public Model[] models { get; set; }
   }
}
